if (typeof CTF == "undefined" || !CTF) {
  var CTF = {};
}

var inconsistencyMessage = "Refresh your view to get the latest changes.";
var showMore = "Show more results...";
var contextualMenuHeight;

function setInconsistencyMessage(message) {
  inconsistencyMessage = message;
}

function setMoreResultMessage(message) {
  showMore = message;
}

function initCTF() {
  // Drag and drop handles for all the elements.
  CTF.DTDrags = {};
  // List of all the artifacts retrieved
  CTF.allArtifacts = {};
  // The cache of all the requests made
  CTF.childArtifactsCache = {};
  // Key: artifact that has children; Value: children artifacts
  CTF.dndArtifactsFamilies = {};
  // List of alien artifacts
  CTF.aliens = {};
  // Number of rows to render
  CTF.numRows = 0;
  // All the functions to rebuild all the move constraints
  CTF.moveConstraints = [];
  // The number of ajax calls in progress
  CTF.DNDAjaxCalls= 0;
  //global counter (for first level artifacts)
  CTF.counter=0;
  //number of pages currently displayed
  CTF.numberPages=0;
  //whether there are additional pages to display
  CTF.hasNextPage=false;
  //data for the tree
  CTF.baseTree = {};
  var jqBrowser = jQuery.browser;

  
  // some older browsers are slower when it comes to dom manipulation, so avoid certain display features for them
  CTF.domLiteOnly = jqBrowser.msie && (jqBrowser.version == "7.0" || jqBrowser.version == "6.0");

  CTF.beforeAndAfterCache = {};
}

function initialLoadURL() {
  return CTF.mode == "rank" ? "/sf/planning/do/viewPlanningFolderRankJSON/" : "/sf/planning/do/viewPlanningFolderSortedJSON/";
}

function childrenLoadURL() {
  return CTF.mode == "rank" ? "/sf/planning/do/listChildrenByRankJSON/" : "/sf/planning/do/listChildrenSortedJSON/";
}

function falseFunction() {
  return false;
}

function isDNDUnLocked() {
  return CTF.DNDAjaxCalls === 0;
}

//some of this vars will be used and updated if needed on 
//table_scroll.js for planning folder sort / rank mode
var ffTableHeaderPostionTop = 0;
var artifactTableScrollCurrentPosition = 0;
var isChildRender = false;

function renderArtifacts(data, insertAt, before, idPrefix, folderPath, display, nextPageNumber, cacheKey) {

  CTF.beforeAndAfterCache = {};
  var referenceCell = $j(insertAt);
  var rowNum;

  if (CTF.dndArtifactsFamilies[idPrefix] === undefined) {
    CTF.dndArtifactsFamilies[idPrefix] = [];
  }

  var counter = 0;
  var fistLevel = false;
  if (idPrefix == 'row_') {
    counter = CTF.counter;
    firstLevel = true;
  }
  var toExpand = [];
  for (rowNum in data.values) {
    var row = data.values[rowNum].row;
    var artifactId = data.values[rowNum].artifactId;
    var rowId = idPrefix + artifactId;
    var isAlien = data.values[rowNum].isAlien;
    CTF.allArtifacts[idPrefix + artifactId] = true;
    if (display) {
      CTF.dndArtifactsFamilies[idPrefix][counter] = idPrefix + artifactId;
      counter++;
      var domRow = document.createElement("tr");
      domRow.setAttribute("class", "tbl-row treeview");
      // Workaround for IE6 and IE7
      domRow.setAttribute("className", "treeview");
      var cellNum;
      var domText;
      var numRows = 0;
      for (cellNum in row) {
        var cell = row[cellNum];
        var domCell = document.createElement("td");
        if (cell.type == "checkbox") {
          ++numRows;
          var ck = $j("<input>").attr("name", cell.name).attr("type", "checkbox");
          if (cell.onclick !== undefined) {
            ck.click(new Function("eval(\"" + cell.onclick + "\")"));
          }
          ck.attr("value", cell.value);
          if (cell.headers !== undefined) {
            domCell.setAttribute("headers", cell.headers);
          }
          if (cell.title !== undefined) {
            ck.attr("title", cell.title);
          }
          if (cell.classCkName !== undefined) {
        	  ck.attr("class", cell.classCkName);
        	  ck.attr("className", cell.classCkName); // for older IE
          }
          if (cell.id !== undefined) {
        	  ck.attr("id", cell.id);
          }
          var lb = $j("<label>").attr("for", cell.id).attr("class", "custom-label").attr("name", "labelFor_listItem");
          ck.appendTo(domCell);
          lb.appendTo(domCell);
        } else if (cell.type == "text") {
          ++numRows;
          domText = document.createTextNode(cell.value);
          if (cell.align !== undefined) {
            domCell.setAttribute("align", cell.align);
          }
          if (cell.className !== undefined) {
              domCell.setAttribute("class", cell.className);
              domCell.setAttribute("className", cell.className); // for older IE
          }
          if (cell.headers !== undefined) {
            domCell.setAttribute("headers", cell.headers);
          }
          if (cell.link !== undefined) {
            var domA = document.createElement("a");
            domA.appendChild(domText);
            domA.setAttribute("href", cell.link);
            domA.setAttribute("target", "_self");
            domCell.appendChild(domA);
          } else {
            domCell.appendChild(domText);
          }
        } else if (cell.type == "image") {
          ++numRows;
          var domImage = document.createElement("img");
          if (cell.src !== undefined) {
            domImage.setAttribute("src", cell.src);
          }
          if (cell.border !== undefined) {
            domImage.setAttribute("border", cell.border);
          }
          if (cell.idName !== undefined) {
            domImage.setAttribute("id", cell.idName);
          }
          if (cell.title !== undefined) {
            domImage.setAttribute("title", cell.title);
            domImage.setAttribute("alt", cell.alt);
          }
          if (cell.headers !== undefined) {
            domCell.setAttribute("headers", cell.headers);
          }
          domCell.appendChild(domImage);
        } else if (cell.type == "html") {
          ++numRows;
          if (cell.className !== undefined) {
            domCell.setAttribute("class", cell.className);
            domCell.setAttribute("className", cell.className); // for older IE
          }
          if (cell.headers !== undefined) {
            domCell.setAttribute("headers", cell.headers);
          }
          domCell.innerHTML = cell.value;
        } else {
          ++numRows;
          domText = document.createTextNode(cell.value);
          domCell.appendChild(domText);
        }
        
        
        if (typeof allowedToEditColumnFields == 'function') {
	      //inline edit, add the edit class for editable columns
	        if(allowedToEditColumnFields(cell.headers)){
	        	
	        	$j(domCell).addClass("edit");
	        	
	        	if (isNumericField(cell.headers)) {
	        		$j(domCell).addClass("numeric");
	        	}
	        }
        
        }
        
        domRow.appendChild(domCell);
      }
      CTF.numRows = numRows;
      domRow.setAttribute("id", rowId);
      if (before) {
        referenceCell.before(domRow);
      } else {
        referenceCell.after(domRow);
        referenceCell = $j("#" + rowId);
      }
      //Close the contextual menu if the artifact loses the hovered state. 
      $j(domRow).hover(function(){
             // 
      }, function(){
            $j("li.contextual-menu").removeClass("open"); 
      });
      if (dndEnabled) {
        // Create a new drag and drop handle and store it.
        CTF.DTDrags[rowId] = new CTF.DDRows(rowId, 'dndGroup', { cont : "ArtifactListTable" });
        if (isAlien) {
          // Disable drag for alien artifacts
          CTF.DTDrags[rowId].clickValidator = falseFunction;
          CTF.aliens[rowId] = true;
          domRow.style.cursor="default"; // change cursor to make it appear not 'draggable'
        } else {
          CTF.DTDrags[rowId].clickValidator = isDNDUnLocked;
        }
      }
      domA = null;
      domCell = null;
      domImage = null;
      domRow = null
      domText = null;
    }
    if (isNodeExpanded(rowId, folderPath)) {
      toExpand.push({
        artifactId : artifactId,
        rowId : rowId.substring(4)
      });
    }
  }
  
  var inExpansion;
  for (inExpansion in toExpand) {
    expandNodeTreeView(folderPath, toExpand[inExpansion].artifactId,
        toExpand[inExpansion].rowId, display);
  }
  if (display) {
    rebuildConstraints();
    if (!CTF.domLiteOnly) {
      $j("#ArtifactListTable tr.treeview:odd").removeClass("EvenRow").addClass("OddRow");
      $j("#ArtifactListTable tr.treeview:even").removeClass("OddRow").addClass("EvenRow");
    }
    if (dndEnabled) {
      // remove our "wait" cursor and revert all the rows back to a draggable "move"
      $j("#ArtifactListTable").removeClass("dndWait").addClass("dndEnabled");
    }
    if (firstLevel) {
      CTF.counter = counter;
    }
  }

  if (display && firstLevel && nextPageNumber >= 2) {
    var key = '';
    if (!dndEnabled) {
      key = cacheKey;
    }
    //insert a row showing that there are more items
    $j("#treeFooter").before("<tr id='moreMsg'><td class='showMoreResults' colspan='" + CTF.numRows + "'><a  href=\"\" OnClick=\"javascript:getNextItems(" + nextPageNumber + ",'" + key + "','" + folderPath + "');return false;\">" + showMore + "</a></td></tr>");

  }

  if (display && firstLevel) {
	  if ($j("#editMode") != undefined && CTF.mode.indexOf("sort") == 0 && toExpand.length == 0) {
		  setButtonState("_SfButton_inLineEditModeButton", "enabled");
	  }
  }
  
}

function getNextItems(nextPageNumber, cacheKey, folderPath) {
  $j("#moreMsg").remove();
  $j("#treeFooter").before("<tr id='loadId'><td colspan='" + CTF.numRows + "'><img align='middle' src='/sf-images/masthead/loadingIndicator.gif' alt='loading'/></td></tr>");
  loadPage(folderPath, cacheKey, nextPageNumber);
}

function removeAllTreeElements() {
  // Delete all the rows that have a class = "treeview" as we are going to do a full refresh
  var elements = $j(".treeview");
  elements.each(
    function() {
      var dd = CTF.DTDrags[this.id];
      if (dd != undefined) {
        YAHOO.util.Event.purgeElement(dd.getDragEl());
        if (dd.getDragEl() != undefined && dd.getDragEl().parentNode != undefined) {
          dd.getDragEl().parentNode.removeChild(dd.getDragEl());
        }
        dd.unreg();
        delete CTF.DTDrags[this.id];
      }
      dd = null;
    }
  );
  YAHOO.util.DragDropMgr.refreshCache({dndGroup:true});
  elements.remove();
}

/**
 * Rebuild the d&d constraints for all the rows.
 */
function rebuildConstraints() {
  var r;
  for (r in CTF.moveConstraints) {
    CTF.moveConstraints[r]();
  }
  r = null;
}

// Initialize drag and drop logic
function initDND() {
  var Dom = YAHOO.util.Dom,
    Event = YAHOO.util.Event,
    DDM = YAHOO.util.DragDropMgr;

  // ////////////////////////////////////////////////////////////////////////////
  // Custom drag and drop class
  // ////////////////////////////////////////////////////////////////////////////
  CTF.DDRows = function(id, sGroup, config) {
    this.cont = config.cont;
    CTF.DDRows.superclass.constructor.call(this, id, sGroup, config);
    Dom.addClass(this.getDragEl(), "drag-and-drop-class");
    this.goingUp = false;
    this.lastY = 0;
  };

  // ////////////////////////////////////////////////////////////////////////////
  // DDRows extends DDProxy
  // ////////////////////////////////////////////////////////////////////////////
  YAHOO.extend(
    CTF.DDRows,
    YAHOO.util.DDProxy,
    {
      proxyEl : null,
      srcEl : null,
      srcData : null,
      srcId : null,
      previousArtifact : null,
      nextArtifact : null,
      validDrop : null,
      cont : null,
      init : function() {
        // Call the parent's init method
        CTF.DDRows.superclass.init.apply(this, arguments);
        var remoteThis = this;
        var f = function() {
          remoteThis.initConstraints();
        };
        CTF.moveConstraints.push(f);
      },

      initConstraints : function() {
        // Get the top, right, bottom and left positions
        var region = Dom.getRegion(this.cont);

        // Get the element we are working on
        var el = this.getEl();

        // Get the xy position of it
        var xy = Dom.getXY(el);

        // Get the width and height
        var height = parseInt(Dom.getStyle(el, 'height'), 10);

        // Set top to y minus top
        var top = xy[1] - region.top;

        // Set bottom to bottom minus y minus height
        var bottom = region.bottom - xy[1] - height;

        // Set the constraints based on the above calculations
        this.setXConstraint(0, 0); // locking constraint to Y axis movement only
        this.setYConstraint(top, bottom);
      },

      startDrag : function(x, y) {
        var proxyEl = this.proxyEl = this.getDragEl(),
          srcEl = this.srcEl = this.getEl();

        this.srcId = srcEl.id;
        srcHTML = srcEl.innerHTML;
        // Make the proxy look like the source element
        Dom.setStyle(srcEl, "visibility", "hidden");
        var extraRow = "";
        var childrenRemoved = this.hideAllChildren(this.srcId);
        // If the artifact has children, then we add the "end of page" mock
        if (childrenRemoved != 0) {
          extraRow = "<tr><td id='proxy_eop_id' colspan='" + CTF.numRows + "' style='height:10px;background-color:white;background-image:url(\"/sf-images/tracker/endOfPageRed.png\");background-repeat:x-repeat; font-size: 1px;'>&nbsp</td></tr>";
          // Fix the proxy border
          proxyEl.style.height = "21px";
        }
        YAHOO.util.Dom.setStyle(proxyEl, 'opacity', 0.5);
        proxyEl.innerHTML = "<table id='proxy_id' width='100%' style='background-color:#6c6;'><tbody>"
            + srcEl.innerHTML
            + extraRow
            + "</tbody></table>";
        this.validDrop = false;
        // make all rows show the "busy/wait" cursor so the refresh lag would be less frustrating
        $j("#ArtifactListTable").removeClass("dndEnabled").addClass("dndWait");
        extraRow = null;
        childrenRemoved = null;
        proxyEl = null;
        srcEl = null;
      },

      endDrag : function(x, y) {
        var srcEl = this.srcEl;

        Dom.setStyle(this.proxyEl, "visibility", "hidden");
        this.proxyEl.innerHTML = "";
        Dom.setStyle(srcEl, "visibility", "");
        if (this.validDrop) {
          CTF.DNDAjaxCalls++;
          var handlerURL = "/sf/planning/do/rankArtifactJSON/"
              + globalFolderPath
              + "?fromUrlKey="
              + fromUrlKey
              + "&r="
              + Math.floor(Math.random() * 10000);

          if (CTF.numberPages >= 2) {
            handlerURL += "&_numPages=" + CTF.numberPages;
          }
          var message = {
            "previous" : this.extractId(this.previousArtifact),
            "actual" : this.extractId(this.srcId),
            "next" : this.extractId(this.nextArtifact)
          };

          // alert("previous: " + this.extractId(this.previousArtifact) +
          // "\nactualArtifact: " + this.extractId(this.srcId) +
          // "\nnextArtifact: " + this.extractId(this.nextArtifact));
          //
          $j.getJSON(handlerURL, message, function(data) {
              if (data.action === undefined) {
                CTF.DNDAjaxCalls--;
                // We had an issue.
                alert(inconsistencyMessage);
              } else if (data.action == "render") {
                // We need to remove the elements first as removing them will release the YUI DnD objects
                removeAllTreeElements();
                
                // Kill the cache
                initCTF();
                CTF.baseTree[0] = data;

                //delete the show more results message
                $j("#moreMsg").remove();
                var hasNextPage = data.hasNextPage;
                var truncatedResult = data.truncatedResult;
                CTF.numberPages = data.pageNumber;
                var nextPageNumber = -1;
                if (hasNextPage) {
                  nextPageNumber = CTF.numberPages + 1;
                  CTF.hasNextPage=true;
                }

                renderArtifacts(data, "#treeFooter", true, "row_", globalFolderPath, false, nextPageNumber);
                renderArtifacts(data, "#treeFooter", true, "row_", globalFolderPath, true, nextPageNumber);
                if (truncatedResult) {
                    scrollTo(0,0);
                }
                hasNextPage = null;
                truncatedResult = null;
                nextPageNumber = null;
              } else if (data.action == "refresh") {
                window.location.reload()
              } else {
                CTF.DNDAjaxCalls--;
                // Handle error and other cases.
                alert(inconsistencyMessage);
              }
          });
        } else {
          removeAllTreeElements();
          $j("#moreMsg").remove();
          var nextPageNumber = -1;
          if (CTF.hasNextPage) {
            nextPageNumber = CTF.numberPages + 1;
          }
          CTF.dndArtifactsFamilies = {};
          var r;
          for (r in CTF.baseTree) {

            if (r == CTF.numberPages) {
              renderArtifacts(CTF.baseTree[r], "#treeFooter", true, "row_", globalFolderPath, true, nextPageNumber);
            } else {
              renderArtifacts(CTF.baseTree[r], "#treeFooter", true, "row_", globalFolderPath, true, -1);
            }
          }
          r = null;
        }
      },

      onDrag : function(e) {
        // Keep track of the direction of the drag for use during onDragOver
        var y = Event.getPageY(e);

        if (y < this.lastY) {
          this.goingUp = true;
        } else if (y > this.lastY) {
          this.goingUp = false;
        }
        this.lastY = y;
        y = null;
      },

      onDragOver : function(e, id) {
        // Reorder rows as user drags
        var destEl = Dom.get(id);

        if (destEl.nodeName.toLowerCase() === "tr" && !CTF.aliens[id]) {
          var beforeAndAfter = this.findBeforeAndAfterArtifacts(this.srcId, id);
          if (beforeAndAfter !== null) {
            if (this.goingUp) {
              $j("#" + beforeAndAfter[1]).before(this.srcEl);
              this.previousArtifact = beforeAndAfter[0];
              this.nextArtifact = beforeAndAfter[1];
            } else {
              var lastInDisplay = this.lastInDisplay(beforeAndAfter[1]);
              $j("#" + lastInDisplay).after(this.srcEl);
              this.previousArtifact = beforeAndAfter[1];
              this.nextArtifact = beforeAndAfter[2];
              lastInDisplay = null;
            }
            this.proxyEl.style.cursor = "move";
            this.validDrop = (this.previousArtifact != null) || (this.nextArtifact != null);
          } else {
            this.proxyEl.style.cursor = "not-allowed";
            this.validDrop = false;
          }
          DDM.refreshCache();
          if (!CTF.domLiteOnly) {
            $j("#ArtifactListTable tr.treeview:odd").removeClass("EvenRow").addClass("OddRow");
            $j("#ArtifactListTable tr.treeview:even").removeClass("OddRow").addClass("EvenRow");
          }
          this.validDrop = (this.previousArtifact != null) || (this.nextArtifact != null);

          if (this.validDrop) {
            $j("#proxy_id").css("background-color", "#6c6");
            $j("#proxy_eop_id").css("background-image", "url(\"/sf-images/tracker/endOfPageGreen.png\")");
          } else {
            $j("#proxy_id").css("background-color", "#c66");
            $j("#proxy_eop_id").css("background-image", "url(\"/sf-images/tracker/endOfPageRed.png\")");
          }
          beforeAndAfter = null;
        }
        destEl = null;
      },

      extractId : function(preId) {
        if (preId === null || CTF.aliens[preId]) {
          return null;
        }
        var result = preId.substring(preId.lastIndexOf("_") + 1);
        return result;
      },

      /*
       * For a given id, hides all the children. Return the
       * number of children hidden.
       */
      hideAllChildren : function(id) {
        var result = 0;
        var r;
        for (r in CTF.dndArtifactsFamilies[id + "_"]) {
          var j = $j("#" + CTF.dndArtifactsFamilies[id + "_"][r]);
          result = j.size();
          j.remove();
          result += this.hideAllChildren(CTF.dndArtifactsFamilies[id + "_"][r]);
        }
        DDM.refreshCache();
        return result;
      },

      lastInDisplay : function(id) {
        while (CTF.dndArtifactsFamilies[id + "_"] !== undefined) {
          var a = CTF.dndArtifactsFamilies[id + "_"];
          id = a[a.length - 1];
        }
        return id;
      },

      /*
       * Returns an array with the following information:
       * the array in position 1 has the artifact you are hovering over.
       * at position 0 has the artifact, at the same level, before the artifact at position 1 (or null if there is none)
       * at position 2 has the artifact, at the same level, after the artifact at position 1 (or null if there is none)
       */
      findBeforeAndAfterArtifacts : function(srcId, destId) {
        var key = srcId + "--" + destId;
        if (CTF.beforeAndAfterCache[key] !== undefined) {
          return CTF.beforeAndAfterCache[key];
        }
        var srcParent = this.findParent(srcId);
        var previousParent = this.findParent(destId);
        while (previousParent !== null && previousParent != srcParent) {
          destId = previousParent;
          previousParent = this.findParent(destId);
        }
        if (previousParent === null) {
          // This is an invalid drag and drop
          CTF.beforeAndAfterCache[key] = null;
          return null;
        }
        var result = [];
        result[0] = null;
        result[1] = destId;
        result[2] = null;
        var found = false;
        var c;
        for (c in CTF.dndArtifactsFamilies[previousParent + "_"]) {
          if (found) {
            if (CTF.dndArtifactsFamilies[previousParent + "_"][c] != srcId) {
              result[2] = CTF.dndArtifactsFamilies[previousParent + "_"][c];
              break;
            }
          }
          if (CTF.dndArtifactsFamilies[previousParent + "_"][c] == destId) {
            found = true;
          } else if (CTF.dndArtifactsFamilies[previousParent + "_"][c] != srcId) {
            result[0] = CTF.dndArtifactsFamilies[previousParent + "_"][c];
          }
        }
        c = null;
        CTF.beforeAndAfterCache[key] = result;
        return result;
      },

      /*
       * For a given artifact row id, returns the parent id or
       * null if this is a top level artifact.
       */
      findParent : function(id) {
        var parent = null;
        var p;
        var c;
        for (p in CTF.dndArtifactsFamilies) {
          for (c in CTF.dndArtifactsFamilies[p]) {
            if (CTF.dndArtifactsFamilies[p][c] == id) {
              // Remove the trailing "_"
              parent = p.substring(0, p.length - 1);
            }
          }
        }
        p = c = null;
        return parent;
      }

  });
}




function loadPage(folderPath, cacheKey, pageNumber) {
  CTF.DNDAjaxCalls++;
  // first we get a json response for the output of the expanded rows
  var handlerURL;
  //In case of list artifacts from team
  if (folderPath.indexOf("teamId") !== -1) {
   handlerURL = "/sf/teams/do/viewTeamSortedJSON/" + folderPath + "&fromUrlKey=" + fromUrlKey  + "&r=" + Math.floor(Math.random() * 10000);
  } else {
   handlerURL = initialLoadURL() + folderPath
      + "?fromUrlKey=" + fromUrlKey + "&r="
      + Math.floor(Math.random() * 10000);
  }
  if (cacheKey !== undefined) {
    handlerURL += "&treeDataListKey=" + cacheKey;
  }

  if (pageNumber != null && pageNumber > 0) {
    handlerURL += "&_pagenum=" + pageNumber;
  }



  jQuery(document).ajaxError(function(event, request, settings) {
	
	 // if we are on in line edit mode, skip this
	if (fieldNameIds != null){
		return;
	}

    alert(inconsistencyMessage);
  });

  $j.getJSON(handlerURL, function(data) { 
    if (data.action === undefined) {
      // We had an issue.
      alert(inconsistencyMessage);
    } else if (data.action == "render") {
      //TODO: would be good to find another way to do that since pageNumber is in the closure
      CTF.baseTree[pageNumber] = data;

      if (pageNumber == null || pageNumber == '' || pageNumber <=1) {
        // Delete all the rows that have a class = "treeview"
        $j(".treeview").remove();
      } else {
        //delete only the loading gif
        $j("#loadId").remove();
      }

      CTF.numberPages = data.pageNumber;
      CTF.hasNextPage = data.hasNextPage;
      var nextPageNumber = -1;
      if (CTF.hasNextPage) {
        nextPageNumber = CTF.numberPages + 1;
      }
      renderArtifacts(data, "#treeFooter", true, "row_", folderPath, false, nextPageNumber, cacheKey);
      renderArtifacts(data, "#treeFooter", true, "row_", folderPath, true, nextPageNumber, cacheKey);

      if(typeof setFloatingComponentsData == 'function') {
    	  //this call is for the planning folder view
    	  if(typeof setInLineEditor == 'function') {
    		  setInLineEditor(null);
    	  }
    	  setFloatingComponentsData(true);
    	  doTheHeaderRender();
    	  setInterval("checkWidthAndHeight()" , 100);

      }
    } else {
      alert(inconsistencyMessage);
    }
    CTF.DNDAjaxCalls--;
  });
}

function loadInitialTree(folderPath, cacheKey) {
  initCTF();
  initDND();
  loadPage(folderPath, cacheKey, 1);

}

// Loads the rank tree and displays it in the table
function loadInitialRankTree(folderPath, cacheKey) {
  CTF.mode = "rank";
  loadInitialTree(folderPath, cacheKey);
}

//Loads the rank tree and displays it in the table
function loadInitialSortedTree(folderPath, cacheKey) {
  CTF.mode = "sort";
  loadInitialTree(folderPath, cacheKey);
}


function expandNodeTreeView(folderPath, artfId, rowId, display) {
  if (display === undefined) {
    display = true;
  }
  if (display) {
    var imgId = "img_" + rowId;
    var image = document.getElementById(imgId);
    if (image == null) {
    	removeNodeFromCookie("row_" + rowId, folderPath);
    } else {
    	
	    image.onclick = function() {
	      collapseNodeTreeView(folderPath, artfId, rowId);
	    };
	    // Change the image
	    image.src = image.src.replace("plus", "minus");
    }

    if (CTF.childArtifactsCache[rowId] !== undefined) {
    	isChildRender = true;
      renderArtifacts(CTF.childArtifactsCache[rowId], "#row_" + rowId,
          false, "row_" + rowId + "_", folderPath, display, -1, null);
      inlineEditChildArtifactRefresh(artfId);
      //Hiding contextual menu while inline mode is enabled
    if(typeof editMode !== "undefined" && editMode){
       hideContextualMenu();
    }
      updateOnResize(true);
      return;
    }

    addNodeToCookie("row_" + rowId, folderPath);
  }
  CTF.DNDAjaxCalls++;

  // Check if we already have this artifact in the page
  var handlerURL = childrenLoadURL() + folderPath
      + "?fromUrlKey=" + fromUrlKey + "&rowId=" + rowId + "&artfId="
      + artfId + "&r=" + Math.floor(Math.random() * 10000);
  $j.ajax( {
    url : handlerURL,
    dataType : 'json',
    success : function(data) {
      if (data.action === undefined) {
        // We had an issue.
        alert(inconsistencyMessage);
      } else if (data.action == "render") {
        CTF.childArtifactsCache[rowId] = data;
        isChildRender = true;
        renderArtifacts(data, "#row_" + rowId, false, "row_" + rowId + "_", folderPath, display, null);
        isChildRender = false;
        inlineEditChildArtifactRefresh(artfId);
      } else {
        // Handle error and other cases.
        alert(inconsistencyMessage);
      }
      CTF.DNDAjaxCalls--;
    },
    async : false
  });
//Hiding contextual menu while inline mode is enabled
   if(typeof editMode !== "undefined" && editMode){
	hideContextualMenu();
   }
  updateOnResize(true);
  
}

function inlineEditChildArtifactRefresh(artfId) {
	if(typeof handleChildArtifactOnPlanningFolder == 'function') {
    		  handleChildArtifactOnPlanningFolder(artfId);
    }
} 

/**
 * Hide the contextual menu
 */
function hideContextualMenu(){
    $j("li.contextual-menu").attr("style", "visibility: hidden");
}

/**
 * if user wants to collapse a child and it has changed let him know and decide
 * wheter to keep it expanded or losing the changes.
 * @param artfId
 * */
function userReallyWantsToCollapseRow(artfId) {
	var defaultResponse = true;
	
	if(typeof handleChildArtifactOnPlanningFolder == 'function') {
    		  if (editMode) {
    			  
    			  if (artifactHasEditedChildrens(artfId)) {
    				  
    				  var userResponse = window.confirm(COLLAPSE_CHILD_ALERT);
    				  
    				  if (userResponse) {
    					  removeArtifactsThatWereEditedAndAreBeingCollapsed(artfId);
    				  }
    				  
    				  return userResponse;
    			  }
    		  }
    }
	return defaultResponse;
} 

function collapseNodeTreeView(folderPath, artfId, rowId) {
  
  //only for inline edit, alert user before collapsing the node	
  if (!userReallyWantsToCollapseRow(artfId)) {
	  return;
  }
  
  var imgId = "img_" + rowId;
  var image = document.getElementById(imgId);
  image.onclick = function() {
    expandNodeTreeView(folderPath, artfId, rowId);
  };
  image.src = image.src.replace("minus", "plus");
  // Remove the children
  var artifactId;
  var toRemove = {};
  for (artifactId in CTF.allArtifacts) {
    if (artifactId.indexOf("row_" + rowId + "_") == 0) {
      $j("#" + artifactId).remove();
      toRemove[artifactId] = true;
    }
  }
  removeNodeFromCookie("row_" + rowId, folderPath);
  delete CTF.dndArtifactsFamilies["row_" + rowId + "_"];
  for (artifactId in toRemove) {
    delete CTF.allArtifacts[artifactId];
  }
  if (!CTF.domLiteOnly) {
    $j("#ArtifactListTable tr.treeview:odd").removeClass("EvenRow").addClass("OddRow");
    $j("#ArtifactListTable tr.treeview:even").removeClass("OddRow").addClass("EvenRow");
  }
  updateOnResize(true);
}

var expandedNodes = {};
var expandedNodesInited = false;

function addNodeToCookie(id, folderPath) {
  if (!expandedNodesInited) {
    initExpandedNodes(folderPath);
  }
  expandedNodes[id] = true;
  storeCookie(expandedNodes, folderPath);
}

function removeNodeFromCookie(id, folderPath) {
  if (!expandedNodesInited) {
    initExpandedNodes(folderPath);
  }
  var node;
  for (node in expandedNodes) {
    if (node.indexOf(id) == 0) {
      delete expandedNodes[node];
    }
  }
  storeCookie(expandedNodes, folderPath);
}

function isNodeExpanded(id, folderPath) {
  if (!expandedNodesInited) {
    initExpandedNodes(folderPath);
  }
  return expandedNodes[id];
}

function initExpandedNodes(folderPath) {
  expandedNodesInited = true;
  var cookieValues = readCookie(folderPath);
  if (cookieValues === null) {
    return;
  }
  var nodes = cookieValues.split("&");
  var node;
  for (node in nodes) {
    expandedNodes[nodes[node]] = true;
  }
}

function storeCookie(nodes, folderPath) {
  var expirationDate = new Date();
  expirationDate.setDate(expirationDate.getDate() + 1);
  var cookieValue = [];
  var node;
  for (node in nodes) {
    cookieValue.push(node);
  }
  document.cookie = folderPath + "=" + cookieValue.join("&") + ";expires="
      + expirationDate.toGMTString() + ";path=/sf";
}

function readCookie(name) {
  var nameEQ = name + "=";
  var ca = document.cookie.split(';');
  for ( var i = 0; i < ca.length; i++) {
  var c = ca[i];
  while (c.charAt(0) === ' ') {
    c = c.substring(1, c.length);
  }
  if (c.indexOf(nameEQ) == 0)
    return c.substring(nameEQ.length, c.length);
  }
  return null;
}

function removeParentArtifact(artifactPath, parentArtifactId, returnUrl, submitUrl) {
    var removeForm = document.getElementById('removeParent');
    removeForm.action = "/sf/tracker"+ artifactPath;
    removeForm._listItem.value=parentArtifactId;
    removeForm.followLink.value=returnUrl;
    removeForm._submitterURL.value=submitUrl;
    removeForm.sfsubmit.value="remove";
    removeForm.submit();
}

// Add click event to the contextual menu icon and handle the  collision detection.
function openMenuAboveOrBelow(artifactId) {       
    if (contextualMenuHeight == undefined) {
        contextualMenuHeight = $j("li.contextual-menu ul").height();
    }
    var iconEl = $j("#icon-for-" + artifactId);
    var windowHeight = $j(window).height();
    var artfTop = $j(iconEl).offset().top;
    if ((artfTop + contextualMenuHeight + 40) > (windowHeight + $j(window).scrollTop())) {
        $j(iconEl).parents("li.contextual-menu").addClass("dropup");
    } else {
        $j(iconEl).parents("li.contextual-menu").removeClass("dropup");
    }   
}
